/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "robot_motor_pic_instructions.h" //robot instruction codes
#include "motor.h"  //motor data structures (motor port values and masks, duration, and program data structure)

#include <xc.h>
#include <sys/attribs.h>
#include "system_definitions.h"


#if Linux
#include "osal/osal.h" //for TCPIP_NET_IF
#include "tcpip/src/link_list.h" //for TCPIP_NET_IF
#include "tcpip/src/tcpip_manager_control.h"  //for TCPIP_NET_IF
#endif
#if Win32
#include "osal\osal.h" //for TCPIP_NET_IF
#include "tcpip\src\link_list.h" //for TCPIP_NET_IF
#include "tcpip\src\tcpip_manager_control.h"  //for TCPIP_NET_IF
#endif
        

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;
static char appMsgToClient[] = "Hello Client!\n\r";
static uint8_t appMsgFromClient[80];

static TCPIP_NET_HANDLE    	app_netH;
static SYS_STATUS          	app_tcpipStat;
static int                 	app_nNets;


#define PCB_NAME_LENGTH 5
static const char *PCB_Name = "Motor";
//#define VNH5019A //testing VNH5019A chip
//#define DRV8836 //testing DRV8836 chip
#define DRV8800 //testing DRV8800 chip

//For instructions sent over the network (UDP) (and eventually possibly over USB)
#define MAX_INSTRUCTION_LEN 512 //max of 512 uint8_ts in instruction
#define INSTRUCTION_PORT_NUM    53510 //the port of the MAC sending instructions, and the port of this MAC receiving instructions
//IP_MULTI_ADDRESS remoteip;
UDP_SOCKET UDPSock,UDPSendSock;
uint16_t UDPNumuint8_ts;
uint8_t InstData[MAX_INSTRUCTION_LEN];
uint16_t UDPPortNum=INSTRUCTION_PORT_NUM;
bool UDPSuccess;
//TCPIP_NET_IF  *pMyNetIf;
uint16_t A2DCount; //counts up ADC interrupts
uint32_t LastSample; //last ADC sample sent
uint8_t SenseMotor; //which motor to send the current sense of

//Motor variables
uint8_t NumMotors;
MotorStatus Motor[MAX_NUM_MOTORS];  //status of each motor
StoredInstructions SInst[MAX_NUM_STORED_INSTRUCTIONS];  //stored instructions
uint16_t CurSInst; //current stored instruction
uint16_t NumSInst; //number of stored instructions
uint16_t CurSInstTime; //current stored instruction time (in ms)
uint8_t StoredProgramStatus; //status of stored program - running, paused, stopped
int NumClocksInMotorDutyCycle; //the number of timer interrupt clocks in the motor duty cycle
int MotorDutyCycleClock; //the time (in us) of each timer2 interrupt when the motor pins are updated
uint8_t CSSendBuffer[50]; //Current Sense UDP return data buffer
uint8_t CSSendBufferLen; //length of Current Sense buffer

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary callback functions.
*/

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************

/******************************************************************************
  Function:
    static void TCP_Server_TXRX_Task (void)
    
   Remarks:
    Feeds the USB write function. 
*/
static void TCP_Server_TXRX_Task(void)
{
	static IPV4_ADDR    		dwLastIP[2] = { {-1}, {-1} };
	static IPV4_ADDR           	ipAddr;
	int                 		i;
    uint16_t wMaxGet;

	switch (appData.txrxTaskState)
	{
        case APP_TCPIP_WAIT_FOR_IP:
        {
            app_nNets = TCPIP_STACK_NumberOfNetworksGet();

            for (i = 0; i < app_nNets; i++)
            {
                app_netH = TCPIP_STACK_IndexToNet(i);
                ipAddr.Val = TCPIP_STACK_NetAddress(app_netH);
                if (TCPIP_STACK_NetIsReady(app_netH))
                {                
                    if(dwLastIP[i].Val != ipAddr.Val)
                    {                         
                         //SYS_CONSOLE_MESSAGE(TCPIP_STACK_NetNameGet(app_netH));
                         //SYS_CONSOLE_MESSAGE(" IP Address: ");
                         //SYS_CONSOLE_PRINT("%d.%d.%d.%d \r\n", ipAddr.v[0], ipAddr.v[1], ipAddr.v[2], ipAddr.v[3]);                         
                         
                         dwLastIP[i].Val = ipAddr.Val;
                         
                    } //if(dwLastIP[i].Val != ipAddr.Val)                

                    appData.txrxTaskState = APP_TCPIP_OPENING_SERVER;
                } //if (TCPIP_STACK_NetIsReady(app_netH))
            }
            break;
        }
        case APP_TCPIP_OPENING_SERVER:
        {
            //appData.socket = TCPIP_TCP_ServerOpen(IP_ADDRESS_TYPE_IPV4, appData.port, 0);
            appData.socket = TCPIP_UDP_ServerOpen(IP_ADDRESS_TYPE_IPV4, appData.port, 0);
            if (appData.socket == INVALID_SOCKET)                
            {
                SYS_CONSOLE_PRINT("Failed to open UDP Socket %d\r\n",appData.port);
                break;
            }
            appData.txrxTaskState = APP_TCPIP_WAIT_FOR_CONNECTION;;
        }
        break;

        case APP_TCPIP_WAIT_FOR_CONNECTION:
        {
            //if (!TCPIP_TCP_IsConnected(appData.socket))
            if (!TCPIP_UDP_IsConnected(appData.socket))
            {
                break;
            }
            else
            {
                // We got a connection
				//TCPIP_TCP_ArrayPut(appData.socket, appMsgToClient, sizeof(appMsgToClient));
                //appData.txrxTaskState = APP_TCPIP_WAIT_FOR_RESPONSE;
                appData.txrxTaskState = APP_TCPIP_SERVING_CONNECTION;
            }
        }
        break;

        case APP_TCPIP_SERVING_CONNECTION:
        {
            if (!TCPIP_UDP_IsConnected(appData.socket))
            {
                appData.state = APP_TCPIP_CLOSING_CONNECTION;
                SYS_CONSOLE_MESSAGE("Connection was closed\r\n");
                break;
            }
            //if any data has been received process it
            //In theory there could be more or less than one complete
            //instruction, but instructions come one in a packet
            //and are only a few bytes, and this loop checks for new data
            //so rapidly, that a single instruction is
            //always read at once.
            //todo: probably there should be code to check for 
            //a second instruction or extra data at the end of a 
            //received UDP buffer.
            
            wMaxGet = TCPIP_UDP_GetIsReady(appData.socket);	// Get UDP RX FIFO byte count

            if (wMaxGet>0) {
                Process_Instruction(wMaxGet);
            }

//          appData.state = APP_TCPIP_CLOSING_CONNECTION;
//          TCPIP_UDP_Discard(appData.socket);
        }
        break;

        case APP_TCPIP_CLOSING_CONNECTION:
        {
         	// Close the socket connection.
            TCPIP_UDP_Close(appData.socket);

            appData.state = APP_TCPIP_OPENING_SERVER;

        }
        break;

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}

/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;

	appData.port = 53510;
    
    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
    
    InitializeMotors();
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    int i;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
        {
            bool appInitialized = true;
       
            app_tcpipStat = TCPIP_STACK_Status(sysObj.tcpip);
            if(app_tcpipStat < 0)
            {   // some error occurred
                appData.state = APP_STATE_ERROR;
				appInitialized = false;
            }
            else if(app_tcpipStat == SYS_STATUS_READY)
            {
                // now that the stack is ready we can check the
                // available interfaces
                app_nNets = TCPIP_STACK_NumberOfNetworksGet();
                for(i = 0; i < app_nNets; i++)
                {
                    app_netH = TCPIP_STACK_IndexToNet(i);
                }
                appData.txrxTaskState = APP_TCPIP_WAIT_FOR_IP;
            }
        
            if (appInitialized)
            {
            
                appData.state = APP_STATE_SERVICE_TASKS;
            }
            break;
        }

        case APP_STATE_SERVICE_TASKS:
        {
            TCP_Server_TXRX_Task();
        
            break;
        }

        /* TODO: implement your application state machine.*/
        
        case APP_STATE_ERROR:
        {
			break;
		}

        /* The default state should never be executed. */
        default:
        {
            /* TODO: Handle error in application's state machine. */
            break;
        }
    }
}


//Process any kind of instruction
void Process_Instruction(uint16_t InstDataLen)
{
    uint32_t* MemAddr;
    uint32_t MemData;
    uint32_t InstValue;
    uint8_t *SendData;
    uint8_t ReturnInst[50]; //currently just 50 bytes but probably will change
    uint32_t InstLen;
    TCPIP_NET_HANDLE netH;
    uint32_t ReturnInstLen;
    TCPIP_NET_IF *pNetIf;
    uint32_t NumBytes;

    //the first 4-bytes of the instruction are the IP of the source of the instruction
    //this IP needs to be sent back so the return data can reach the correct requester
    //in particular when the instruction involves a long term periodic sending back of data

    // Transfer the data out of the TCP RX FIFO and into our local processing buffer.
    NumBytes= TCPIP_UDP_ArrayGet(appData.socket, InstData, InstDataLen);

    //SYS_CONSOLE_PRINT("\tReceived %d bytes\r\n",NumBytes);    
    
    //switch(SetupPkt.bRequest)  //USB
    //switch(InstData[0]) //Robot Instruction
    switch(InstData[4]) //Robot Instruction
    {
    case ROBOT_MOTORS_TEST: //send back 0x12345678
        memcpy(ReturnInst,InstData,5); //copy IP + inst byte to return instruction
        ReturnInst[6]=0x12;
        ReturnInst[7]=0x34;
        ReturnInst[8]=0x56;
        ReturnInst[9]=0x78;
        while (TCPIP_UDP_PutIsReady(appData.socket)<9) {};
        TCPIP_UDP_ArrayPut(appData.socket,ReturnInst,9);  //little endian       
        TCPIP_UDP_Flush(appData.socket); //send the packet        
        //while (UDPIsTxPutReady(UDPSendSock,9)<9) {};
        //UDPPutArray(UDPSendSock,(uint8_t *)ReturnInst,9);  //little endian
        //UDPFlush(UDPSendSock); //send the packet
        break;
    case ROBOT_MOTORS_PCB_NAME: //01 send back mac+name/id
        memcpy(ReturnInst,InstData,5); //copy IP + inst byte to return instruction
        ReturnInstLen=5;
        //get the MAC address from the default network interface
        //this presumes that there is only 1 net
        //in the future there could be more than 1 net, 
        //for example a wireless net too
        //netH = TCPIP_STACK_GetDefaultNet();
        app_netH = TCPIP_STACK_IndexToNet(0);  //presumes net 0 is the wired net
        //pMyNetIf = _TCPIPStackHandleToNet(netH);
        pNetIf = _TCPIPStackHandleToNetUp(app_netH);
        memcpy(ReturnInst+ReturnInstLen,(pNetIf)->netMACAddr.v,sizeof(pNetIf->netMACAddr));//copy mac
        ReturnInstLen+=sizeof(pNetIf->netMACAddr);
        memcpy(ReturnInst+ReturnInstLen,PCB_Name,PCB_NAME_LENGTH);//copy name
        ReturnInstLen+=PCB_NAME_LENGTH; //MOTOR
        //while (UDPIsTxPutReady(UDPSendSock,ReturnInstLen)<ReturnInstLen) {};
        //UDPPutArray(UDPSendSock,(uint8_t *)ReturnInst,ReturnInstLen);  //little endian
        //UDPFlush(UDPSendSock); //send the packet
        while (TCPIP_UDP_PutIsReady(appData.socket)<ReturnInstLen) {};
        TCPIP_UDP_ArrayPut(appData.socket,ReturnInst, ReturnInstLen);  //little endian       
        TCPIP_UDP_Flush(appData.socket); //send the packet

        break;
    case ROBOT_MOTORS_GET_MEM: //get PIC memory value
        MemAddr=(uint32_t *)((InstData[8]<<0x18)|(InstData[7]<<0x10)|(InstData[6]<<0x8)|InstData[5]);
        memcpy(ReturnInst,InstData,5); //copy IP + inst byte to return instruction
        memcpy(ReturnInst+5,MemAddr,4); //little endian
        while (TCPIP_UDP_PutIsReady(appData.socket)<8) {};
        TCPIP_UDP_ArrayPut(appData.socket,ReturnInst,8);  //little endian       
        TCPIP_UDP_Flush(appData.socket); //send the packet
    break;
    case ROBOT_MOTORS_SET_MEM: //set PIC memory value
        //write to PIC memory
        //presumes 2 32-bit little-endian integer values
        MemAddr=(uint32_t *)((InstData[8]<<0x18)|(InstData[7]<<0x10)|(InstData[6]<<0x8)|InstData[5]);
        MemData=(uint32_t)((InstData[12]<<0x18)|(InstData[11]<<0x10)|(InstData[10]<<0x8)|InstData[9]);
#if 0 //currently disabled 
        *MemAddr=MemData;
#endif
    break;
        case ROBOT_MOTORS_SEND_4BYTE_INST:  //got send a 4 byte Instruction over Net (was USB)
        case ROBOT_MOTORS_SEND_PROGRAM_INST_STARTTIME:
        case ROBOT_MOTORS_SEND_PROGRAM_INST: //save stored prog instruction
        case ROBOT_MOTORS_NEW_PROGRAM: //start to load a new set of stored instructions
        case ROBOT_MOTORS_RUN_PROGRAM: //run the stored program
        case ROBOT_MOTORS_STOP_PROGRAM: //stop the stored program
        case ROBOT_MOTORS_STEP_PROGRAM: //step thru the stored program
        case ROBOT_MOTORS_PAUSE_PROGRAM: //pause the stored program
        case ROBOT_MOTORS_CONTINUE_PROGRAM: //resume running the stored program
            Process_4Byte_Instruction(); //process a 3 uint8_t instruction
            //ProcessB24Inst=1;  //process this instruction later
            //inPipes[0].info.bits.busy = 1;   // This tells USBCtrlEPServiceComplete()
            //otherwise we get an error 9 from libusb- can't just SendB24(SetupPkt);
        break;
	case ROBOT_MOTORS_GET_4BYTE_DATA:  //request for data send 4 Byte Instruction over USB
            //get data returned from sent addr and data uint8_t thru 4 Byte Inst
    break;
    case ROBOT_MOTORS_GET_MOTOR_DUTY_CYCLE_CLK_IN_USEC:  
        //send back SourceIP+data so data is sent to correct requesting machine
        memcpy(ReturnInst,InstData,5); //copy IP + inst byte to return instruction
        //default is: ROBOT_MOTORS_DEFAULT_MOTOR_DUTY_CYCLE_CLK 25 //25us 40khz above human hearing range
        memcpy(ReturnInst+5,&MotorDutyCycleClock,4); //little endian
        while (TCPIP_UDP_PutIsReady(appData.socket)<9) {};
        TCPIP_UDP_ArrayPut(appData.socket,ReturnInst,9);  //little endian       
        TCPIP_UDP_Flush(appData.socket); //send the packet            
        //while (UDPIsTxPutReady(UDPSendSock,9)<9) {};
        //UDPPutArray(UDPSendSock,ReturnInst,9);
        //UDPFlush(UDPSendSock); //send the packet
    break;
    case ROBOT_MOTORS_SET_MOTOR_DUTY_CYCLE_CLK_IN_USEC:
        //set the motor duty cycle clock interrupt in microseconds
        //has 1 parameter=4 uint8_t Little Endian int
        InstValue=(uint32_t)((InstData[8]<<0x18)|(InstData[7]<<0x10)|(InstData[6]<<0x8)|InstData[5]);
        //InstValue=*(uint32_t *)&InstData[1];  //doesn't work
        if (InstValue!=MotorDutyCycleClock) {
            MotorDutyCycleClock=InstValue;
            //T2CONbits.ON=0; //disable the timer- need or else intrpt doesn't change interval
            //IEC0bits.T2IE=0; //disable the Timer 2 interrupt
            
            //convert from 1 us to PBCLK/8 100ns units

            //duty cycle must also be divided by 2 
            //so that "on" pulses which produce sound will 
            //be at 40khz.
            //in order to set the on and off transitions
            PR2=((MotorDutyCycleClock*10)>>1)&0xffff;
            //32-bit high 16-bits:
            PR3=(((MotorDutyCycleClock*10)>>1)&0xffff0000)>>0x10;
            TMR2=0;
            TMR3=0;


            //IPC2bits.T2IP=4;//.INT2IP=4;  //priority
            //IPC2bits.T2IS=2;//.INT2IS=2;  //subpriority
            //IEC0bits.T2IE=1;        //enable the Timer 2 interrupt
            //T2CONbits.ON=1; //enable the timer

            //IFS0bits.T2IF=0; //clear the timer2 interrupt flag
        }
    break;
    case ROBOT_MOTORS_GET_NUM_CLK_IN_MOTOR_DUTY_CYCLE:
        //while (!UDPIsTxPutReady(UDPSendSock,1)) {};
        memcpy(ReturnInst,InstData,5); //copy IP + inst byte to return instruction
        memcpy(ReturnInst+5,&NumClocksInMotorDutyCycle,4); //little endian
        while (TCPIP_UDP_PutIsReady(appData.socket)<9) {};
        TCPIP_UDP_ArrayPut(appData.socket,ReturnInst,9);  //little endian       
        TCPIP_UDP_Flush(appData.socket); //send the packet            

//        while (UDPIsTxPutReady(UDPSendSock,8)<8) {}; //add timout?
//            UDPPutArray(UDPSendSock,(uint8_t *)&ReturnInst,9);  //little endian
 //           UDPFlush(UDPSendSock); //send the packet
    break;
    case ROBOT_MOTORS_SET_NUM_CLK_IN_MOTOR_DUTY_CYCLE:
        //set the number of clock ticks in each motor duty cycle (default=7)
        //has 1 parameter=4 uint8_t Little Endian int
        InstValue=(uint32_t)((InstData[8]<<0x18)|(InstData[7]<<0x10)|(InstData[6]<<0x8)|InstData[5]);
        if (InstValue!=NumClocksInMotorDutyCycle) {
            NumClocksInMotorDutyCycle=InstValue;
        }
    break;
    case ROBOT_MOTORS_SEND_CURRENT_SENSE:
        memcpy(CSSendBuffer,InstData,5); //copy IP + inst byte to return instruction
        CSSendBufferLen=5;
        //decode the rest of the instruction if any (which motor#s)
        SenseMotor=0xff;
        if (InstDataLen>1) { //there is a motor number
            //send the sample for this motor
            //SenseMotor=InstData[1];  //presumes motor 1 = 0x00
            SenseMotor=InstData[5];  //presumes motor 1 = 0x00
        } //if (InstDataLen>1) { //there is a motor number
        LastSample=0;  //set the last sample to 0
        IEC1bits.AD1IE=1; //enable AD1 interrupt
    break;
    case ROBOT_MOTORS_STOP_SENDING_CURRENT_SENSE:
        IEC1bits.AD1IE=0; //disable AD1 interrupt
    break;
    default:
        //inPipes[0].info.bits.busy = 1;   // This tells USBCtrlEPServiceComplete()
        //otherwise we get an error 9 from libusb
    break;
    }//end switch
}  //Process_Instruction


//Process 4 uint8_t instructions
void Process_4Byte_Instruction(void) 
{
uint8_t i;
uint8_t MotorInst[4];

//B24Addr=RXPkt.wValue;  //(uint16_t)((InstData[1]<<0x8)+InstData[2])
//B24Data=RXPkt.wIndex;  //InstData[3]

//switch(RXPkt.bRequest) {
//switch(InstData[0]) {
switch(InstData[4]) {
    case ROBOT_MOTORS_SEND_4BYTE_INST: //execute a motor instruction now
        //SendMotorInst(B24Addr,B24Data);
        //MotorInst[0]=InstData[1];
        //MotorInst[1]=InstData[2];
        //MotorInst[2]=InstData[3];
        MotorInst[0]=InstData[5];  //Motor Num<<4
        MotorInst[1]=InstData[6];  //Dir+Strength
        MotorInst[2]=InstData[7];  //duration low byte
        MotorInst[3]=InstData[8];  //duration high byte
        SendMotorInst(MotorInst);
    break;
    case ROBOT_MOTORS_NEW_PROGRAM: //start to load a new set of stored instructions
        NumSInst=0;
        CurSInst=0;
        CurSInstTime=0;
        StoredProgramStatus=0;
    break; //case ROBOT_SEND_PROGRAM_INST_STARTTIME:
    case ROBOT_MOTORS_SEND_PROGRAM_INST_STARTTIME: //save stored prog inst start time
        //SInst[NumSInst].StartTime=(uint16_t)((InstData[2]<<0x8)+InstData[1]);
        SInst[NumSInst].StartTime=(uint16_t)((InstData[6]<<0x8)+InstData[5]);
        //convert to timer interrupts
       //duration is currently stored in timer interrupts- which depend on duty cycle
        SInst[NumSInst].StartTime=(SInst[NumSInst].StartTime*2000)/MotorDutyCycleClock;
        //currently only uses 16-bits so data is ignored
        //still waiting for instruction
    break; //case ROBOT_SEND_PROGRAM_INST_STARTTIME:
    case ROBOT_MOTORS_SEND_PROGRAM_INST: //save stored prog instruction
        //memcpy(SInst[NumSInst].Inst,&InstData[1],3);
        //SInst[NumSInst].Inst[0]=InstData[1];
        //SInst[NumSInst].Inst[1]=InstData[2];
        //SInst[NumSInst].Inst[2]=InstData[3];
        SInst[NumSInst].Inst[0]=InstData[5];
        SInst[NumSInst].Inst[1]=InstData[6];
        SInst[NumSInst].Inst[2]=InstData[7];
        //have starttime and instruction so increment to number of stored insts
        NumSInst++;
    break; //case ROBOT_SEND_PROGRAM_INST_STARTTIME:
    case ROBOT_MOTORS_RUN_PROGRAM: //run the stored program
        //if already running just ignore for now
        StoredProgramStatus=STORED_PROGRAM_RUNNING;
        CurSInst=0;
        CurSInstTime=0;
    break; //case ROBOT_RUN_PROGRAM:
    case ROBOT_MOTORS_STOP_PROGRAM: //stop the stored program
        //ignore if no stored program is running?
        StoredProgramStatus=0;
        //for now turn off all motors in case this was an emergency interrupt
        LATB=0;
        for(i=0;i<NumMotors;i++) {
            Motor[i].Duration=0;
        } //for i
        CurSInst=0;
        CurSInstTime=0;
    break; //case ROBOT_STOP_PROGRAM:
    case ROBOT_MOTORS_STEP_PROGRAM: //step thru the stored program
        //ignore if no stored program is running?
        StoredProgramStatus=STORED_PROGRAM_STEPPING;
    break; //case ROBOT_STEP_PROGRAM:
    case ROBOT_MOTORS_PAUSE_PROGRAM: //pause the stored program
        //ignore if no stored program is running?
        StoredProgramStatus=STORED_PROGRAM_PAUSED;
    break; //case ROBOT_PAUSE_PROGRAM:
    case ROBOT_MOTORS_CONTINUE_PROGRAM: //resume running the stored program
        StoredProgramStatus=STORED_PROGRAM_RUNNING;
    break; //case ROBOT_CONTINUE_PROGRAM

} //switch(InstData[0]) {

} //Process_4Byte_Instruction


//this instruction is a motor instruction to execute now
//and contains (in 4 bytes):
//Motor# (0:3),unused(4:7),direction(8),thrust(9:15),duration(16:31)
//was 3 bytes:
//motor# (address):4, direction:1, strength:3,duration:16
void SendMotorInst(uint8_t *MInst)
{
    uint8_t MotorNum,Strength;
    uint32_t Duration;

    //set the motor status so the timer interrupt
    //will find that a motor has a duration and needs to be moved
    //determine which motor
    MotorNum=(MInst[0]&0xf0)>>4;

    Motor[MotorNum].Duration=0; //just in case motor is currently running (pulsing)
    SYS_TMR_DelayMS(1);  //wait to make sure motor is stopped if running (pulsing)
    Motor[MotorNum].flags|=MOTOR_INST_FIRST_RUN; //first set motor pins to 00 to avoid short circuit
    //for example, motor is turning with 10 then a 01 instruction is sent, and for a nanosecond when LATB is being set, possibly 2 pins might be 1 and cause a short at the h-bridge.

    //if ((MInst[0]&0x08)!=0) {
    if ((MInst[1]&0x80)!=0) {
        //counter-clockwise
       // Motor[MotorNum].DirectionMask=0x40>>(MotorNum*2);  //counter-clockwise
         //Motor[MotorNum].DirectionMask=0x1<<(MotorNum*2);  //counter-clockwise
         Motor[MotorNum].Direction=MOTOR_COUNTERCLOCKWISE;
    } else {
        //clock-wise
        //Motor[MotorNum].DirectionMask=0x80>>(MotorNum*2);  //clockwise
        //Motor[MotorNum].DirectionMask=0x2<<(MotorNum*2);  //clockwise
        Motor[MotorNum].Direction=MOTOR_CLOCKWISE;
    }

    //set motor turn strength- number of timer clocks in "on" part of duty cycle
    //0=stop 0x7=full speed (NumClkInDutyCycle)
    //this number if multiplied by the NumClocksInMotorDutyCycle/7
    //so for strength=1, Strength*NumClocksInMotorDutyCycle/7 (1 clks of 7 are on)
    //for strength=7 the motor is on for every clock (7 of 7)
    //Motor[MotorNum].Strength=((MInst[0]&0x07)*NumClocksInMotorDutyCycle)/ROBOT_MOTORS_DEFAULT_NUM_CLKS_IN_MOTOR_DUTY_CYCLE;
    Strength=(MInst[1]&0x7f);//max is currently 0 to 127
    //if strength is higher than the number of possible speeds, set at maximum strength
    if (Strength> ROBOT_MOTORS_DEFAULT_NUM_CLKS_IN_MOTOR_DUTY_CYCLE) {
        Strength=ROBOT_MOTORS_DEFAULT_NUM_CLKS_IN_MOTOR_DUTY_CYCLE;
    }

    //set motor turn duration (is 16-bit little endian int)
    //Motor[MotorNum].Duration=(uint16_t)((MInst[2]<<8)+MInst[1]);
    Duration=(uint32_t)((MInst[3]<<8)+MInst[2]);
    //duration from user is in ms, so convert to timer interrupts:
    //Motor[MotorNum].Duration is in timer interrupts 
    //MotorDutyCycleClock is 25us by default (40khz) /2 = every 12.5us
    //so convert Duration into time interrupt units (how many 12.5us units)

    //if Duration*1000 < 127*MotorDutyCycleClock then Strength needs to be scaled down
    //because the entire duty cycle of the motor will be < 127.
    //ex: DurTime=1ms, MDCC=25us DurInts=1000/25=only 40 so if strength=127/2=63, that needs to be scaled down to 63/127=x/40
    //x=63*40/127=19 so 19 timer interrupts will be on, and 21 off for a total of 40x25us=1ms

    //convert ms to 12.5us units
    //so Duration of 100ms in 25uS units=0.1/0.000025=4000 clocks * 2= 8000
    //in us: (100)*1000/25=4000 * 2 8000, so generalizing in us:
    //Duration*2000/MotorDutyCycle = number of TimerInterrupts for 
    //user sent Duration in ms.
    Duration*=2000; //Duration is divided by MotorDutyCycleClock below
    //MotorDutyCycleClock is in ms so multiply Duration x 1000 and 
    //x2 for on+off here.
/*
 //    
    if (Duration/MotorDutyCycleClock < NumClocksInMotorDutyCycle) {  //(Duration/25<127)
        //strength needs to be scaled down 
        //at 40khz (25us) a pulse:
        //1/10 speed gives a duty cycle of 127*25us=3.175ms, 
        //so if the motor duration is < 3.175ms, the strength needs to be
        //scaled down.
        //(probably this will not ever happen since the minimum reaction time of the robot is currently 10ms 
        //which is 100000/25=4000 timer interrupts. Note that at 1/2 speed the PWM would be 63on 63off x 3
        //but not all pwm will work out perfectly, for example for duration = 3ms: 3000/25= 120 so 63on but only 57 off
        //- although the motor is then off after. 
        // with 127 speeds: 1/10 speed is 12on 115off = 300us on  2875us off
        // with   7 speeds: 1/10 speed is  1on   6off =  25us on   150us off 
        //so probably a lower number of speeds is better, for example: 20-40 speeds
        Strength=(Strength*Duration)/(NumClocksInMotorDutyCycle*MotorDutyCycleClock);
        //ex: 63*1000/127*25 = 19 timer interrupts of the 1000/25=40 total duty cycle interrupts
    }
*/
    Motor[MotorNum].Strength=Strength;
    Motor[MotorNum].StrengthCount=0;

    
    //Convert duration in us to duration in number of timer interrupts
    Motor[MotorNum].Duration=Duration/MotorDutyCycleClock;  //note: this is the equivalent of enabling the motor
    Motor[MotorNum].DurationCount=0;

    //start timer3 if not already started
    if (T2CONbits.ON==0) {
            T2CONbits.ON=1;//enable timer2 - done at start causes problems with usb connection
    //put initial count into timer0 to cause an interrupt
            TMR2=0;
            TMR3=0;
    }

} //SendMotorInst



//Set initial settings for this device
int InitDevice(void)
{

    /* Configure the target for maximum performance at 80 MHz */
    //SYSTEMConfigPerformance(80000000UL);
    DDPCONbits.JTAGEN = 0;                  // This turns off the JTAG and allows PORTB pins to be used
    DDPCONbits.TROEN=0;  //turn off trace enable bits


 /*
//8888MUST BE BEFORE RESET- because lATF=0 resets PhY (nRST)
        //TRISA=0;
    TRISC=0;
    TRISD=0;
    TRISE=0;
    TRISF=0;
    TRISG=0;

    //set them all to 0
    //LATA=0;
    LATC=0;
    LATD=0;
    LATE=0;
    LATF=0;
    LATG=0;

*/


    TRISFbits.TRISF0=0;  //set nRST as output
    //SYS_TICK_MsDelay(1);  //min=100us
    SYS_TMR_DelayMS(1); 
    LATFbits.LATF0=0;
    //SYS_TICK_MsDelay(1);  //wait before sending anything for the PHY to initialize
    SYS_TMR_DelayMS(1); 
    LATFbits.LATF0=1;

//#if 0
//    TRISB=0xffff; //configure port B as input

    //16 motors - set all ports as outputs
    //(except portB which is used for current sense)


    
    //turn off prefetch
    //see http://www.microchip.com/forums/m637174-print.aspx
    //I am getting a lot of DBE exceptions during interrupts
    //When both prefetch and instruction cache are
//enabled, a Data Bus Exception (DBE) may occur if
//an interrupt is encountered by the CPU while it is
//accessing data from Flash memory.
//    CHECONbits.PREFEN=0;
    //Disable Ethernet interrupts in the EVIC by clearing bit ETHIE
    //EVICbits.

   //LATB=0x5555;  //set REGB to 0x5555
   //LATB=0xffff;  //set REGB to 0xffff
    //LATB=0xaaaa;  //set REGB to 0xffff
//    LATB=0x0;  //set REGB to 0x0

    //tph do hardware reset on PHY nRST- active low
    //TRISB=0;  //configure port B as output
//    AD1PCFG=0xff; //set PORTB pins (also analog pins) to all digital, (all AN# pins are analog by default)
    AD1PCFG=0x00; //set PORTB pins (also analog pins) to all analog, (all AN# pins are analog by default)
    //TRISDbits.TRISD11=0;  //configure port D pin 0 as output- was using for testing
    //TRISBbits.TRISB14=0;  //configure port B pin 14 as output
    //TRISFbits.TRISF0=0;  //configure port F pin 0 as output (PHY nReset)
    //TRISAbits.TRISA0=0;  //configure port A pin 0 as output (PHY nReset)
    //TRISFbits.TRISF4=0;  //configure port F pin 4 as output (pin12 on 10x2 header)
    //TRISFbits.TRISF5=0;  //configure port F pin 5 as output (pin10 on 10x2 header)
 

    //LATBbits.LATB14=0;  //set port B pin 14 to 0 (1 indicates got data in test mode)
    //LATB=0;



 
//    SYSTEMConfig(GetSystemClock(), SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    //send UDP dest IPs
    //remoteip.v4Add.Val=0x5f00a8c0; //192.168.0.95 in Little Endian, 0x5f00a8c0
    //remoteip.v4Add.Val=0x7000a8c0; //192.168.0.112 in Little Endian, 0x5f00a8c0

    // configure for multi-vectored mode
    //INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);
    // enable interrupts
    //INTEnableInterrupts();

     //asm volatile("ei"); //enable interrupts

    //Disable external interrupts- I found that IFS0bits.INT0IF and INT4IF were set w/o this
    //I found that even though external interrupts are disabled by default, the IFS0bits.INT0If and INT4IF flags are set at the timer2 interrupt
    //INTCON&=0xffe0; //disable external interrupts
    //IFS0=0; //clear all interrupt flags
    //I think interrupt flags are apparently just set even if the interrupt is not enabled

    //set number of clocks in motor duty cycle
    //is 7 (but was 14), 7 timer2 interrupts make 1 full motor duty cycle
    NumClocksInMotorDutyCycle=ROBOT_MOTORS_DEFAULT_NUM_CLKS_IN_MOTOR_DUTY_CYCLE;
    MotorDutyCycleClock=ROBOT_MOTORS_DEFAULT_MOTOR_DUTY_CYCLE_CLK;
    
    //note that Timer1 is used in system_random.c to produce a random number
    //initialize Timer 2 count for motor PWM
    T2CONbits.ON=0; //clear the ON control bit to disable the timer
    //T1CONbits.TCS=0; //select the internal PBCLK source
    //select timer input clock prescale
    //because the peripheral clock is 1:1 with the SYSCLK it is 80MHz (12.5ns)
    //see: #pragma config FPBDIV   = DIV_1         // Peripheral Clock divisor
    //so for 40MHz: (25ns)
    //prescale of 8=200ns (5,000  0x1388)
    //for 80MHz:
    //prescale of 8=100ns (10,000  0x2710 ffff-2710=0xd8ef
    //prescale of 64=800ns
    //prescale of 256=3.2us
    T2CONbits.TCKPS=0x3; //prescale of 1:8  10MHz 100ns
    //ROBOT_DEFAULT_MOTOR_DUTY_CYCLE_CLK
    //MotorDutyCycleClock is in us
    //convert MotorDutyCycleClock from 1ms to 100ns intervals (*10) divided by 2
    //TimerInterrupts should occur every 1/40khz 25us/2  12.5us
    //for example 40,000 pulses per second will cause a sound above human hearing
    //So the timer needs to actually be twice as fast.
    //So divide timer counter by two, so that pulse on+off=40,000 on pulses in 1 second.
    PR2=((MotorDutyCycleClock*10)>>1)&0xffff;
    //32-bit high 16-bits:
    PR3=(((MotorDutyCycleClock*10)>>1)&0xffff0000)>>0x10;

    //PR2=0x1388;
    //confirmed with oscilloscope:
    //0x1388=1ms  (5000 x 200ns)
    //0x2710=2ms
    //0x4e20=4ms
    //0x9c40=8ms
    IFS0bits.T3IF=0; //clear the timer 2 interrupt flag
    //tph- thought this was timer2 IPC2bits.INT2IP=1;  //priority
    //tph -thought this was timer 2  IPC2bits.INT2IS=2;  //subpriority
    //important: interrupt priority must match isr priority
    //IPC2bits.T2IP=4;//4;//.INT2IP=4;  //priority - 3 bits
    //IPC2bits.T2IS=3;//.INT2IS=2;  //subpriority -2 bits
    //T3IP=6,3- will not work I don't know why, but 4,3 works
    IPC3bits.T3IP=4;//4;//.INT2IP=4;  //priority - 3 bits
    IPC3bits.T3IS=1;//.INT2IS=2;  //subpriority -2 bits
    T2CONbits.T32=1;  //32 bit timer
    IEC0bits.T3IE=1;        //enable Timer 2+3 interrupt- doesn't enable the timer
    T2CONbits.ON=1; //enable timer2
    //INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR); //configure multi vector
    //INTEnableSystemMultiVectoredInt(); //enable interrupts
    //ConfigIntTimer1(T1_INT_ON | T2_INT_PRIOR_3 | T2_INT_SUB_PRIOR_3);
    //ConfigIntTimer1(T2_INT_PRIOR_3 | T2_INT_SUB_PRIOR_3);
    //INTCONbits.MVEC=1; //interrupt controller configured for Multi-vector mode


    //set motor pins and initialize ADC
    InitializeMotors();

//NOTE THAT DEBUGGING CAN'T WORK WHEN SETTING MOTOR 0 pins 0 and 1
/*
    CurSInst=0;
    NumSInst=0;
    CurSInstTime=0;
    memset(SInst,sizeof(StoredInstructions)*MAX_NUM_STORED_INSTRUCTIONS,0);
    StoredProgramStatus=0;
*/
//#endif

    return(1);
} //InitDevice

void InitializeMotors(void) {
    uint8_t i,j;
    uint8_t NumLower;

#if defined(VNH5019A)
    NumMotors=2;
    //Clear the robot status array
    memset(Motor,sizeof(MotorStatus)*NumMotors,0);

    //IMPORTANT- DO NOT USE BIT 0 or 1 BECAUSE THE DEBUGGER USES THOSE!
    //Motor 1 - note that pin#s are PORT bit (not header pin# or port bit number) and start with 1

    Motor[0].DirCWPort=&LATF;
    Motor[0].DirCCWPort=&LATF;
    Motor[0].PulsePort=&LATA;
    Motor[0].DirCWPortTRIS=&TRISF;
    Motor[0].DirCCWPortTRIS=&TRISF;
    Motor[0].PulsePortTRIS=&TRISA;
    Motor[0].DirectionCWBit=0x2000;//RF13//0x80;//rg7 0x02;  //rb1   0x100 rb8=for testing
    Motor[0].DirectionCCWBit=0x1000;//RF12//0x100;//rg8 0x01; //rb0 0x80 rb7=for testing
    Motor[0].PulseBit=0x2;//RA1// 0x01; //rb0 same for rev1 and rev2
    Motor[0].CurrentSenseBit=0x40;//RB6//0x04;//0x04 rb2(rev1) 0x02; //rb1(rev2)  0x10; //rb4

    Motor[1].DirCWPort=&LATF;
    Motor[1].DirCCWPort=&LATF;
    Motor[1].PulsePort=&LATF;
    Motor[1].DirCWPortTRIS=&TRISF;
    Motor[1].DirCCWPortTRIS=&TRISF;
    Motor[1].PulsePortTRIS=&TRISF;
    Motor[1].DirectionCWBit=0x20;//RF5
    Motor[1].DirectionCCWBit=0x8;//RF3
    Motor[1].PulseBit=0x10;//RF4
    Motor[1].CurrentSenseBit=0x80;//RB7

    //AD1CHSbits.CH0SA=0; //clear which analog pins are sampled-ignored in scan mode
    for(i=0;i<NumMotors;i++) {
        Motor[i].flags|=MOTOR_DRIVER_USES_PULSE_PIN;

        Motor[i].DirectionCWBitMask=~Motor[i].DirectionCWBit;
        Motor[i].DirectionCCWBitMask=~Motor[i].DirectionCCWBit;
        Motor[i].PulseBitMask=~Motor[i].PulseBit;
        Motor[i].CurrentSenseBitMask=~Motor[i].CurrentSenseBit;
        Motor[i].CSPort=&LATB;
        Motor[i].CSPortTRIS=&TRISB;

        //set direction pins as outputs
        *Motor[i].DirCWPortTRIS&=Motor[i].DirectionCWBitMask;
        *Motor[i].DirCCWPortTRIS&=Motor[i].DirectionCCWBitMask;
        *Motor[i].PulsePortTRIS&=Motor[i].PulseBitMask;
        //set CS pin as input
        *Motor[i].CSPortTRIS|=Motor[i].CurrentSenseBit;
        //set motor direction and pulse pins to 0
        *Motor[i].DirCWPort&=Motor[i].DirectionCWBitMask;
        *Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask;
        *Motor[i].PulsePort&=Motor[i].PulseBitMask;

        AD1PCFG&=~Motor[i].CurrentSenseBitMask;    //make the CS pin analog- done above but just to be sure
        //by setting its corresponding bit in the AD1PCFG register to 0
        //Set the analog pin as the Positive Input Select bits for Sample A Multiplexer Setting

        //convert pin bit mask to pin number, ex: 0x10 (16) to 0x04 (rb4)
        //perhaps there is a one-line command to do this?
        j=0;
        while((Motor[i].CurrentSenseBit>>(j+1))>0) {
            j++;
        }
        //AD1CHSbits.CH0SA|=j; //set which channel to sample- ignored in scan mode
        //note that the sample buffer number does not correspond to the
        //analog pin number- for example, if only AN6 is enabled
        //the first sample in the scan will be in ADC1BUF0 (not ADC1BUF6)
        //and so, if the next motor, motor[1] uses AN3, the ADC buffer for motor[0]
        //will change to ADC1BUF1- so for this reason all pins are sampled
        //and then the ADC1BUF corresponds to the pin number,
        //alternatively, earlier motors could be adjusted to the correct ADC1BUF address
        AD1CSSLbits.CSSL|=Motor[i].CurrentSenseBit; //which analog inputs are scanned
    } //for i

    //go back through motors and
    //determine which ADC1BUF sample the CS sample for each motor will be in
    //Motor[i].CSADCBuf=&ADC1BUF0;

    for(i=0;i<NumMotors;i++) {
        NumLower=0; //number of motors higher than the current motor
        for(j=0;j<NumMotors;j++) {
            if (i!=j && Motor[j].CurrentSenseBit<Motor[i].CurrentSenseBit) {
                NumLower++;
            }
        } //for j
        Motor[i].CSADCBuf=&ADC1BUF0+NumLower*4; //add 4 bytes (+0x10) for each motor below this motor
    } //for i
    //AD1CSSLbits.CSSL=0xffff; //which analog inputs are scanned- currently all

    //Initialize and enable the Analog to Digital module

    //The ADC module writes to the ADC1BUF0..ADC1BUFF buffers starting with
    //ADC1BUF0 until the number of samples set by the SMPI bits is reached
    //probaby the motor inputs should be ordered so that the Motor 1 port
    //corresponds to RB0, Motor 2 to RB1, etc. because now I need to scan
    //up to the highest number RB pin used- but it's probably not a big deal

    //the ADC1BUF0 virtual address is: BF80 90B0
    //maximum=0x3ff (1023) so 1 bit=3mV
    //according to Pololu the CS pin voltage is around 140mV/amp of output current
    //if true, 0x2f (47)=1A, 0x1d3 (467) 1.4v=10A, 2.8v=20As

    //Set VREFL as the Negative Input Select bit for Sample A Multiplexer Setting
    AD1CHSbits.CH0NA=0;
    AD1CON1bits.FORM=4; //integer 32-bit format (10-bits)
    AD1CON1bits.SSRC=7; //(autoconvert) internal counter ends sampling and starts conversion
    //ASAM=1 is critical for getting a constant supply of samples
    AD1CON1bits.ASAM=1; //sampling begins immediately after last conversion
    AD1CON2bits.VCFG=0; //VREFH=VDD VREFL=VSSs
    AD1CON2bits.CSCNA=1; //scan input mode is enabled
    //AD1CON2bits.SMPI=0; //interrupt for each sample
    AD1CON2bits.SMPI=NumMotors; //interrupt after NumMotors samples (which corresponds to one for each motor since they are sampled in order)
    AD1CON2bits.BUFM=0; //buffer configured as 1 16-bit uint16_t buffer
    AD1CON2bits.ALTS=0; //always use Sample A input multiplexer settings
    AD1CON3bits.ADRC=0; //clock derived from the Peripheral Bus Clock (PBCLK)
    //a TAD is an ADC clock cycle- one for each bit plus 2 mores
    AD1CON3bits.SAMC=25;//20;//sample time= 20TAD (12 TAD is typical for a conversion)
    AD1CON3bits.ADCS=0xff;//2; //clock prescaler=2 8TPB=TAD
    //(tph: presumably 8 PBCLK=1 TAD 10MHz 100ns)
    //TAD min=65ns
    //100ns*12=1.2us  100ns*20=2us
    //can be every millisecond
    //but since it doesn't slow the program counter and instructions
    //I don't think it matters if it's very fast
    //and fast means the latest current is always there
    AD1CON1bits.ON=1; //turn on ADC module

    //the result will be in ADC1BUF4 (RB4)
    //AD1 - ADC1 Convert Done interrupt

    IFS1bits.AD1IF=0; //clear AD1 interrupt flag
    IPC6bits.AD1IP=4;  //set interrupt priority
    IPC6bits.AD1IS=2;  //set interrupt subpriority
    //IEC1bits.AD1IE=1; //enable AD1 interrupt

#elif defined(DRV8836) //5 robot arm motors

    //add motors
    NumMotors=5;

    //Clear the robot status array
    memset(Motor,sizeof(MotorStatus)*NumMotors,0);

    //IMPORTANT- DO NOT USE BIT 0 or 1 BECAUSE THE DEBUGGER USES THOSE!
    //Motor 1 - note that pin#s are PORT bit (not header pin# or port bit number) and start with 1
    for(i=0;i<NumMotors;i++) {
        //all pins for all motors are on LATB (PORTB)
        Motor[i].DirCWPort=&LATB;
        Motor[i].DirCCWPort=&LATB;
//        Motor[i].PulsePort=&LATB;
//       Motor[i].CSPort=&LATB;
        Motor[i].DirCWPortTRIS=&TRISB;
        Motor[i].DirCCWPortTRIS=&TRISB;
//        Motor[i].PulsePortTRIS=&TRISB;
//        Motor[i].CSPortTRIS=&TRISB;

        Motor[i].DirectionCCWBit=0x100>>(2*i);
        Motor[i].DirectionCCWBitMask=~Motor[i].DirectionCCWBit;
        Motor[i].DirectionCWBit=0x200>>(2*i);
        Motor[i].DirectionCWBitMask=~Motor[i].DirectionCWBit;

        //set direction pins as outputs
        *Motor[0].DirCWPortTRIS&=Motor[0].DirectionCWBitMask;
        *Motor[0].DirCCWPortTRIS&=Motor[0].DirectionCCWBitMask;
//        *Motor[0].PulsePortTRIS&=Motor[0].PulseBitMask;
        //set CS pin as input
//        *Motor[0].CSPortTRIS|=Motor[0].CurrentSenseBit;

    //set motor direction and pulse pins to 0
    *Motor[i].DirCWPort&=Motor[0].DirectionCWBitMask;
    *Motor[i].DirCCWPort&=Motor[0].DirectionCCWBitMask;
    //*Motor[i].PulsePort&=Motor[0].PulseBitMask;



    } //for i

#elif defined(DRV8800) //
    NumMotors=12;
    //Clear the robot status array
    memset(Motor,sizeof(MotorStatus)*NumMotors,0);

    //IMPORTANT- DO NOT USE BIT 0 or 1 BECAUSE THE DEBUGGER USES THOSE!
    //Motor 1 - note that pin#s are PORT bit (not header pin# or port bit number) and start with 1

//note in EthMotors_DRV8800 MODE is hardwired to 1 (high- slow decay mode- brake)
//mode=0 is fast decay mode (outputs are reversed:
//from datasheet:     This decay mode is equivalent to a phase change where the opposite
//drivers are switched on. When in fast decay, the motor current is not allowed to go
//negative (direction change). Instead, as the current approaches zero, the drivers turn off.

//in my testing- fast decay didn't seem to work as well as slow decay (brake) at quickly stopping the motor
    //the motor shaft didn't stop quickly and kept kind of slowly moving- but I didn't do a lot of testing


    Motor[0].DirCWPort=&LATF;
    //Motor[0].DirCCWPort=&LATF; //not used
    Motor[0].PulsePort=&LATD;
    Motor[0].DirCWPortTRIS=&TRISF;
    //Motor[0].DirCCWPortTRIS=&TRISF; //not used
    Motor[0].PulsePortTRIS=&TRISD; //Enable (PWM)
    Motor[0].DirectionCWBit=0x02;//PHASE (Direction) RF1
    //Motor[0].DirectionCCWBit=0x02;//not used
    Motor[0].PulseBit=0x80;//RD7
    Motor[0].CurrentSenseBit=0x4000;//RB14

    Motor[1].DirCWPort=&LATD;
    Motor[1].PulsePort=&LATD;
    Motor[1].DirCWPortTRIS=&TRISD;
    Motor[1].PulsePortTRIS=&TRISD;
    Motor[1].DirectionCWBit=0x40;//Phase RD6
    Motor[1].PulseBit=0x20;//Enable (PWM) RD5
    Motor[1].CurrentSenseBit=0x2000;//RB13

    Motor[2].DirCWPort=&LATD;
    Motor[2].PulsePort=&LATD;
    Motor[2].DirCWPortTRIS=&TRISD;
    Motor[2].PulsePortTRIS=&TRISD;
    Motor[2].DirectionCWBit=0x10;//Phase RD4
    Motor[2].PulseBit=0x8;//Enable (PWM) RD3
    Motor[2].CurrentSenseBit=0x1000;//RB12

    Motor[3].DirCWPort=&LATD;
    Motor[3].PulsePort=&LATC;
    Motor[3].DirCWPortTRIS=&TRISD;
    Motor[3].PulsePortTRIS=&TRISC;
    Motor[3].DirectionCWBit=0x4;//Phase RD2
    Motor[3].PulseBit=0x4000;//Enable (PWM) RC14
    Motor[3].CurrentSenseBit=0x800;//RB11

    Motor[4].DirCWPort=&LATC;
    Motor[4].PulsePort=&LATD;
    Motor[4].DirCWPortTRIS=&TRISC;
    Motor[4].PulsePortTRIS=&TRISD;
    Motor[4].DirectionCWBit=0x2000;//Phase RC13
    Motor[4].PulseBit=0x1;//Enable (PWM) RD0
    Motor[4].CurrentSenseBit=0x400;//RB10

    Motor[5].DirCWPort=&LATD;
    Motor[5].PulsePort=&LATD;
    Motor[5].DirCWPortTRIS=&TRISD;
    Motor[5].PulsePortTRIS=&TRISD;
    Motor[5].DirectionCWBit=0x800;//Phase RD11
    Motor[5].PulseBit=0x400;//Enable (PWM) RD10
    Motor[5].CurrentSenseBit=0x200;//RB9

    Motor[6].DirCWPort=&LATD;
    Motor[6].PulsePort=&LATD;
    Motor[6].DirCWPortTRIS=&TRISD;
    Motor[6].PulsePortTRIS=&TRISD; //Enable (PWM)
    Motor[6].DirectionCWBit=0x200;//PHASE (Direction) RD9
    Motor[6].PulseBit=0x100;//RD8
    Motor[6].CurrentSenseBit=0x100;//RB8

    Motor[7].DirCWPort=&LATF;
    Motor[7].PulsePort=&LATF;
    Motor[7].DirCWPortTRIS=&TRISF;
    Motor[7].PulsePortTRIS=&TRISF;
    Motor[7].DirectionCWBit=0x8;//Phase RF3
    Motor[7].PulseBit=0x20;//Enable (PWM) RF5
    Motor[7].CurrentSenseBit=0x80;//RB7

    //CAUTION- careful using this motor if debugging- because PGED_RB0 is used- and would be set to output
    Motor[8].DirCWPort=&LATF;
    Motor[8].PulsePort=&LATB;
    Motor[8].DirCWPortTRIS=&TRISF;
    Motor[8].PulsePortTRIS=&TRISB;
    Motor[8].DirectionCWBit=0x10;//Phase RF4
    Motor[8].PulseBit=0x1;//Enable (PWM) RB0
    Motor[8].CurrentSenseBit=0x40;//RB6

    Motor[9].DirCWPort=&LATB;
    Motor[9].PulsePort=&LATB;
    Motor[9].DirCWPortTRIS=&TRISB;
    Motor[9].PulsePortTRIS=&TRISB;
    Motor[9].DirectionCWBit=0x2;//Phase RB1
    Motor[9].PulseBit=0x4;//Enable (PWM) RB2
    Motor[9].CurrentSenseBit=0x20;//RB5

    Motor[10].DirCWPort=&LATG;
    Motor[10].PulsePort=&LATG;
    Motor[10].DirCWPortTRIS=&TRISG;
    Motor[10].PulsePortTRIS=&TRISG;
    Motor[10].DirectionCWBit=0x200;//Phase RG9
    Motor[10].PulseBit=0x100;//Enable (PWM) RG8
    Motor[10].CurrentSenseBit=0x10;//RB4

    Motor[11].DirCWPort=&LATG;
    Motor[11].PulsePort=&LATG;
    Motor[11].DirCWPortTRIS=&TRISG;
    Motor[11].PulsePortTRIS=&TRISG;
    Motor[11].DirectionCWBit=0x80;//Phase RG7
    Motor[11].PulseBit=0x40;//Enable (PWM) RG6
    Motor[11].CurrentSenseBit=0x8;//RB3


    //AD1CHSbits.CH0SA=0; //clear which analog pins are sampled-ignored in scan mode
    for(i=0;i<NumMotors;i++) {
        Motor[i].flags|=MOTOR_DRIVER_USES_PULSE_PIN;

        Motor[i].DirectionCWBitMask=~Motor[i].DirectionCWBit;
        //Motor[i].DirectionCCWBitMask=~Motor[i].DirectionCCWBit;
        Motor[i].PulseBitMask=~Motor[i].PulseBit;
        Motor[i].CurrentSenseBitMask=~Motor[i].CurrentSenseBit;
        Motor[i].CSPort=&LATB;  //todo: don't need in mem because doesn't change
        Motor[i].CSPortTRIS=&TRISB;  //doesn't change

        //set direction pins as outputs
        *Motor[i].DirCWPortTRIS&=Motor[i].DirectionCWBitMask;
        //*Motor[i].DirCCWPortTRIS&=Motor[i].DirectionCCWBitMask;
        *Motor[i].PulsePortTRIS&=Motor[i].PulseBitMask;
        //set CS pin as input
        *Motor[i].CSPortTRIS|=Motor[i].CurrentSenseBit;
        //set motor direction and pulse pins to 0
        *Motor[i].DirCWPort&=Motor[i].DirectionCWBitMask;
        //*Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask;
        *Motor[i].PulsePort&=Motor[i].PulseBitMask;

        AD1PCFG&=~Motor[i].CurrentSenseBitMask;    //make the CS pin analog- done above but just to be sure
        //by setting its corresponding bit in the AD1PCFG register to 0
        //Set the analog pin as the Positive Input Select bits for Sample A Multiplexer Setting

        //convert pin bit mask to pin number, ex: 0x10 (16) to 0x04 (rb4)
        //perhaps there is a one-line command to do this?
        j=0;
        while((Motor[i].CurrentSenseBit>>(j+1))>0) {
            j++;
        }
        //AD1CHSbits.CH0SA|=j; //set which channel to sample- ignored in scan mode
        //note that the sample buffer number does not correspond to the
        //analog pin number- for example, if only AN6 is enabled
        //the first sample in the scan will be in ADC1BUF0 (not ADC1BUF6)
        //and so, if the next motor, motor[1] uses AN3, the ADC buffer for motor[0]
        //will change to ADC1BUF1- so for this reason all pins are sampled
        //and then the ADC1BUF corresponds to the pin number,
        //alternatively, earlier motors could be adjusted to the correct ADC1BUF address
        AD1CSSLbits.CSSL|=Motor[i].CurrentSenseBit; //which analog inputs are scanned
    } //for i

    //go back through motors and
    //determine which ADC1BUF sample the CS sample for each motor will be in
    //Motor[i].CSADCBuf=&ADC1BUF0;

    for(i=0;i<NumMotors;i++) {
        NumLower=0; //number of motors higher than the current motor
        for(j=0;j<NumMotors;j++) {
            if (i!=j && Motor[j].CurrentSenseBit<Motor[i].CurrentSenseBit) {
                NumLower++;
            }
        } //for j
        Motor[i].CSADCBuf=&ADC1BUF0+NumLower*4; //add 4 bytes (+0x10) for each motor below this motor

    //initialize MODE which currently does not change
    //*Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask; //make 0 - fast decay (reverses voltage ob motor)
    //  *Motor[i].DirCCWPort|=Motor[i].DirectionCCWBit; //make 1 - slow decay (brake)
      //LATDbits.LATD6=1;
    } //for i
    //AD1CSSLbits.CSSL=0xffff; //which analog inputs are scanned- currently all

    //*Motor[i].DirCCWPort|=Motor[i].DirectionCCWBitMask; //make 1- slow decay (brake)


    //Initialize and enable the Analog to Digital module

    //The ADC module writes to the ADC1BUF0..ADC1BUFF buffers starting with
    //ADC1BUF0 until the number of samples set by the SMPI bits is reached
    //probaby the motor inputs should be ordered so that the Motor 1 port
    //corresponds to RB0, Motor 2 to RB1, etc. because now I need to scan
    //up to the highest number RB pin used- but it's probably not a big deal

    //the ADC1BUF0 virtual address is: BF80 90B0
    //maximum=0x3ff (1023) so 1 bit=3mV
    //according to Pololu the CS pin voltage is around 140mV/amp of output current
    //if true, 0x2f (47)=1A, 0x1d3 (467) 1.4v=10A, 2.8v=20As

    //Set VREFL as the Negative Input Select bit for Sample A Multiplexer Setting
    AD1CHSbits.CH0NA=0;
    AD1CON1bits.FORM=0; //integer 16-bit format (10-bits)
    AD1CON1bits.SSRC=7; //(autoconvert) internal counter ends sampling and starts conversion
    //ASAM=1 is critical for getting a constant supply of samples
    AD1CON1bits.ASAM=1; //sampling begins immediately after last conversion
    AD1CON2bits.VCFG=0; //VREFH=VDD VREFL=VSSs
    AD1CON2bits.CSCNA=1; //scan input mode is enabled
    //AD1CON2bits.SMPI=0; //interrupt for each sample
    AD1CON2bits.SMPI=NumMotors; //interrupt after NumMotors samples (which corresponds to one for each motor since they are sampled in order)
    AD1CON2bits.BUFM=0; //buffer configured as 1 16-bit uint16_t buffer
    AD1CON2bits.ALTS=0; //always use Sample A input multiplexer settings
    AD1CON3bits.ADRC=0; //clock derived from the Peripheral Bus Clock (PBCLK)
    //a TAD is an ADC clock cycle- one for each bit plus 2 mores
    AD1CON3bits.SAMC=25;//20;//sample time= 20TAD (12 TAD is typical for a conversion)
    AD1CON3bits.ADCS=0xff;//2; //clock prescaler=2 8TPB=TAD
    //(tph: presumably 8 PBCLK=1 TAD 10MHz 100ns)
    //TAD min=65ns
    //100ns*12=1.2us  100ns*20=2us
    //can be every millisecond
    //but since it doesn't slow the program counter and instructions
    //I don't think it matters if it's very fast
    //and fast means the latest current is always there
    AD1CON1bits.ON=1; //turn on ADC module

    //the result will be in ADC1BUF4 (RB4)
    //AD1 - ADC1 Convert Done interrupt

    IFS1bits.AD1IF=0; //clear AD1 interrupt flag
    IPC6bits.AD1IP=4;  //set interrupt priority
    IPC6bits.AD1IS=2;  //set interrupt subpriority
    //IEC1bits.AD1IE=1; //enable AD1 interrupt

#endif //#if defined VNH5019A


}  //InitializeMotors



void __ISR(_ADC_VECTOR,ipl4AUTO)  ADC_ISR(void)   //IPL=Interrupt Priority Level
{
    uint8_t  i,NumSamples;
    uint8_t SendBuffer[32]; //at most 32 bytes (16 10-bit samples as 16-bit #s)
    uint8_t *CurBuffer;
    uint32_t Sample;
    //#define TOUCH_THRESHOLD 11//9
    //9: detects even the tiniest touch, but also detect pressure from motion
    //11: pretty much doesn't detect motion or the tiniest touch, but detects very subtle touches

      //IFS1bits.AD1IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?
    
//    if (A2DCount<625) {  //every 100ms
//       A2DCount++;
//    } else {
        //if (ADC1BUF0+TOUCH_THRESHOLD <LastSample || ADC1BUF0-TOUCH_THRESHOLD>LastSample) {
  //      if (ADC1BUF0-TOUCH_THRESHOLD>LastSample) { //only if adc voltage is larer (pressing down on the sensor- not the sensor recoiling)
            //there is a large enough change in the touch sensor to send a message
  //          A2DCount=0; //reset count
            //if (LastSample>0) { //don't send initial change
  //          if (ADC1BUF0>LastSample) { //just to find the max current
                //send the ADC sample                
                i=SenseMotor;                
                if (SenseMotor==0xff) {
                    i=0;
                    SenseMotor=NumMotors-1;
                }
                NumSamples=SenseMotor-i+1;
                CurBuffer=&CSSendBuffer[CSSendBufferLen];
                while(i<=SenseMotor) {
                    *CurBuffer=i; //motor #
                    CurBuffer++;
                    Sample=*Motor[i].CSADCBuf;
                    memcpy(CurBuffer,(uint16_t *)&Sample,2);
                    i++;
                    CurBuffer+=2;
                    CSSendBufferLen+=3;
                } //while(i<=SenseMotor) {

                //send all the samples at once
                while (TCPIP_UDP_PutIsReady(appData.socket)<CSSendBufferLen) {};
                TCPIP_UDP_ArrayPut(appData.socket,CSSendBuffer,CSSendBufferLen);  //little endian       
                TCPIP_UDP_Flush(appData.socket); //send the packet            
                //while (UDPIsTxPutReady(UDPSendSock,CSSendBufferLen)<CSSendBufferLen) {};
                //UDPPutArray(UDPSendSock,CSSendBuffer,CSSendBufferLen);  //s32-bits little endian
                //UDPFlush(UDPSendSock); //send the packet


                //}
                //for largest current of session until user stops and starts again
                //LastSample=ADC1BUF0; //here slow changes over time may cause a large difference
  //          }//if (LastSample>0) {
            //LastSample=ADC1BUF0; //here slow changes over time may cause a large difference
 //       } //if (ADC1BUF0-TOUCH_THRESHOLD>LastSample) { //only if adc voltage is larer (pressing down on the sensor- not the sensor recoiling)
        //LastSample=ADC1BUF0; //here only large quick changes are noticed

//    }  //if (A2DCount<625) {  //every 100ms

    //currently we are just getting one sample at a time- so disable the ADC interrupt
    IEC1bits.AD1IE=0; //disable AD1 interrupt

    IFS1bits.AD1IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?
}	//void __ISR(_ADC_VECTOR,ipl4) ADC_ISR(void) {  //IPL=Interrupt Priority Level

 

void __ISR(_TIMER_3_VECTOR,ipl4AUTO)  Timer3_ISR(void)   //IPL=Interrupt Priority Level
{
//void __attribute__((interrupt())) _DefaultInterrupt(void) {  //IPL=Interrupt Priority Level


    uint8_t temp;
    uint8_t i;

    //IFS0bits.T3IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?

    //disable other interrupts
 //   INTDisableInterrupts();

    //IFS0bits.T3IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?
#if 0
    //to measure clock
    //LATFbits.LATF5=!LATFbits.LATF5;
    LATBbits.LATB12=!LATBbits.LATB12; //SMT board
//    LATFbits.LATF4=!LATFbits.LATF4;
//    LATDbits.LATD11=!LATDbits.LATD11;

//#if 0
    //check to see if a stored program is running and
    //send to the motors any instructions with the current
    //(or earlier time- but the start time should be an exact match)
    if (StoredProgramStatus&(STORED_PROGRAM_RUNNING|STORED_PROGRAM_STEPPING)) {
        if (CurSInst<NumSInst) {
        //there are still stored instructions to run
        //Note that we also need to check for CurSInst<NumSInst
        //because old instructions are not erased
            while (CurSInst<NumSInst && SInst[CurSInst].StartTime<=CurSInstTime) {
                //run this instruction
                //SendMotorInst(SInst[CurSInst].Address,SInst[CurSInst].Data);
                SendMotorInst(SInst[CurSInst].Inst);
                //advance to the next instruction
                //and see if it needs to be run
                //if stepping, since at least one inst has run
                //put program back to pause until next step command
                //this way no stored program commands will be looked at
                //and stored program time will not accumulate
                //so when a person then resumes with a second step
                //some time may go by until the next step
                if (StoredProgramStatus&STORED_PROGRAM_STEPPING) {
                    StoredProgramStatus=STORED_PROGRAM_PAUSED;
                }
                CurSInst++;
            } //while (SInst[CurSInst]<=CurSInstTime)
        //add another millisecond to the current stored instruction time
        CurSInstTime++;
        } else {//if (CurSInst<NumSInst) {
            StoredProgramStatus=0; //stop running the program
            CurSInst=0; //reset the current instruction
            CurSInstTime=0; //and stored instruction time
        } //if (CurSInst<NumSInst) {
    } //if (StoredProgramStatus&STORED_PROGRAM_RUNNING) {
#endif

    //go through each motor status and set/reset the correct PORTB and PORTC pins
    for(i=0;i<NumMotors;i++) {//NUM_MOTORS;i++) {
        if (Motor[i].Duration>0) { //this motor is moving, or will start moving
            //set strength duty cycle
            if (Motor[i].StrengthCount<Motor[i].Strength && !(Motor[i].flags&MOTOR_INST_FIRST_RUN)) { //set motor pins
                //in "on" portion of duty cycle
                //clear and set the direction pins
                //(is the same for driver with or without a pulse pin)
                if (Motor[i].Direction) {
                    //CCW
                    //clear clockwise pin
                    //LATB&=Motor[i].DirectionCWBitMask;
                    *Motor[i].DirCWPort&=Motor[i].DirectionCWBitMask;
                    //set counter-clockwise pin
                    //LATB|=Motor[i].DirectionCCWBit;
#ifndef DRV8800
                    *Motor[i].DirCCWPort|=Motor[i].DirectionCCWBit;
#endif
                } else {
                    //CW
                    //clear counter-clockwise pin
                    //LATB&=Motor[i].DirectionCCWBitMask;
#ifndef DRV8800
                    *Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask;
#endif
                    //set clockwise pin
                    //LATB|=Motor[i].DirectionCWBit;
                    *Motor[i].DirCWPort|=Motor[i].DirectionCWBit;
                }//if (Motor[i].Direction) {

                //pulse the pulse pin if necessary
                if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
                    //LATB|=Motor[i].PulseBit;
                    *Motor[i].PulsePort|=Motor[i].PulseBit;
                }//if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
            } else {  //if (Motor[i].StrengthCount<Motor[i].Strength)
                //in "off" part of duty cycle (or first run of a motor instruction)

                if (Motor[i].flags&MOTOR_INST_FIRST_RUN) {
                    Motor[i].flags&=~MOTOR_INST_FIRST_RUN; //clear first run bit
                    if (Motor[i].StrengthCount>0) {
                        Motor[i].StrengthCount--; //set back 1 so direction can get set above
                    }
                } //if (Motor[i].flags&MOTOR_INST_FIRST_RUN) {
                
                //if this motor driver uses a pulse pin set pulse pin to 0
                if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
                    //Note that there is no need to set direction pins
                    //because the "off" duty cycle is never called before the "on"
                    //duty cycle, and the "on" duty cycle sets the direction
                    //which doesn't need to change for the "off" cycle
                    //LATB&=Motor[i].PulseBitMask;
                    *Motor[i].PulsePort&=Motor[i].PulseBitMask;
                } else {//if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
                    //the motor driver does not use a pulse pin
                    //so we need to set the direction pin to 0
                    //it's just as fast to clear both and
                    //not check to see what the current direction is
                    //LATB&=Motor[i].DirectionCCWBitMask;
#ifndef DRV8800
                    *Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask;
#endif
                    //LATB&=Motor[i].DirectionCWBitMask;
                    *Motor[i].DirCWPort&=Motor[i].DirectionCWBitMask;
                } //if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {

            } //if (Motor[i].StrengthCount<motor[i].Strength)

            Motor[i].StrengthCount++;  //increase duty cycle count
//because the L293 can only handle a constant 600mA (but non-rep is 1.2a)
//the top speed should only be 7 of 14 ms
            //if (Motor[i].StrengthCount>7) {  //reached end of duty cycle, reset count
            //for example if NumCLocksInMotorDutyCycle=7 0-6 are on and then reset
            if (Motor[i].StrengthCount>=NumClocksInMotorDutyCycle) {  //reached end of duty cycle, reset count
                    Motor[i].StrengthCount=0;
            }

           // Motor[i].DurationCount++;  //increase interupt count
            //if (Motor[i].DurationCount>9) {  //reached 10 interrupts, decrement Duration (in centiseconds) and reset count
                    //Motor[i].Duration--;  //currently units are timer interrupts
              //      Motor[i].DurationCount=0;
           // }
            //decrement Duration
            if (Motor[i].Duration!=0) { //new inst might set Duration=0
                Motor[i].Duration--;  //currently units are timer interrupts
            }
        //which depend on MotorDutyCycleClock
        } else {  //no motor Duration - set motor pins to 0

            if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
                //LATB&=Motor[i].PulseBitMask;
                *Motor[i].PulsePort&=Motor[i].PulseBitMask;
            } else {//if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
                //LATB&=Motor[i].DirectionCCWBitMask;
#ifndef DRV8800
                *Motor[i].DirCCWPort&=Motor[i].DirectionCCWBitMask;
#endif
                //LATB&=Motor[i].DirectionCWBitMask;
                *Motor[i].DirCWPort&=Motor[i].DirectionCWBitMask;
            } //if (Motor[i].flags&MOTOR_DRIVER_USES_PULSE_PIN) {
        } //if (Motor[i].Duration>0)
    } //for(i=0;i<NUM_MOTORS;i++)
//#endif


    IFS0bits.T3IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?
    //INTEnableInterrupts();
   // IFS0=0; //clear all interrupt flags
  //  } //if (IFS0bits.T2IF) { //timer2 interrupt
        //End Timer3 interrupt

}	//void __ISR(_TIMER_3_VECTOR,IPL3AUTO) Timer3_ISR(void) {  //IPL=Interrupt Priority Level



/*******************************************************************************
 End of File
 */

