/*******************************************************************************
 System Interrupts File

  File Name:
    system_interrupt.c

  Summary:
    Raw ISR definitions.

  Description:
    This file contains a definitions of the raw ISRs required to support the
    interrupt sub-system.

  Summary:
    This file contains source code for the interrupt vector functions in the
    system.

  Description:
    This file contains source code for the interrupt vector functions in the
    system.  It implements the system and part specific vector "stub" functions
    from which the individual "Tasks" functions are called for any modules
    executing interrupt-driven in the MPLAB Harmony system.

  Remarks:
    This file requires access to the systemObjects global data structure that
    contains the object handles to all MPLAB Harmony module objects executing
    interrupt-driven in the system.  These handles are passed into the individual
    module "Tasks" functions to identify the instance of the module to maintain.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2011-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <xc.h>
#include <sys/attribs.h>
#include "app.h"
#include "system_definitions.h"
#include "accel.h" //for external interrupts
#include "touchsensors.h"

extern uint8_t IntAccel[5];
extern AccelStatus Accel[MAX_NUM_ACCEL];
extern uint32_t TouchSensorSendLen; 
extern uint8_t NumActiveTouchSensors;
extern uint8_t ActiveTouchSensor[MAX_NUM_TOUCH_SENSORS]; //list of all active touch sensors in order, for a quick reference
extern TouchSensorStatus TouchSensor[MAX_NUM_TOUCH_SENSORS]; //status of each touch sensor
extern EthAccelStatus EAStatus; //status of EthAccelTouch PCB
extern APP_DATA appData;
extern uint8_t TouchSensorSend[TOUCH_SENSOR_SEND_SIZE];  //touch sensor packet data to send back to requester
// *****************************************************************************
// *****************************************************************************
// Section: System Interrupt Vector Functions
// *****************************************************************************
// *****************************************************************************
void __ISR(_EXTERNAL_0_VECTOR, IPL4AUTO) _IntHandlerExternalInterruptInstance0(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_EXTERNAL_0);
    //Accel[IntAccel[0]].flags|=ACCEL_STATUS_GOT_INTERRUPT;
    Accel[IntAccel[0]].flags|=ACCEL_STATUS_CLEAR_INTERRUPT;
    //SYS_CONSOLE_PRINT("i0=a0\r\n");    
}
void __ISR(_EXTERNAL_1_VECTOR, IPL4AUTO) _IntHandlerExternalInterruptInstance1(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_EXTERNAL_1);
    //Accel[IntAccel[1]].flags|=ACCEL_STATUS_GOT_INTERRUPT;
    Accel[IntAccel[1]].flags|=ACCEL_STATUS_CLEAR_INTERRUPT;
    //SYS_CONSOLE_PRINT("i1=a2\r\n");    
}
void __ISR(_EXTERNAL_4_VECTOR, IPL4AUTO) _IntHandlerExternalInterruptInstance2(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_EXTERNAL_4);
    //Accel[IntAccel[4]].flags|=ACCEL_STATUS_GOT_INTERRUPT;
    Accel[IntAccel[4]].flags|=ACCEL_STATUS_CLEAR_INTERRUPT;
    //SYS_CONSOLE_PRINT("i4=a1\r\n");
}

    
void __ISR(_TIMER_1_VECTOR, ipl5AUTO) IntHandlerDrvTmrInstance0(void)
{
    DRV_TMR_Tasks(sysObj.drvTmr0);
}
    
void __ISR(_TIMER_3_VECTOR, ipl3AUTO) IntHandlerDrvTmrInstance1(void)
{
     //DRV_TMR_Tasks(sysObj.drvTmr1);
    
    //Every TimerInterval, process:
    //Accel Polling- get (and send) samples from all enabled and polling accelerometers
    //Accel Interrupt- get (and send) samples from all enabled and interrupting accelerometers
    //Touch Sensor Polling and Interrupting
    IFS0bits.T3IF=0; //clear interrupt flag
    
    if (EAStatus.flags&ETHACCEL_STATUS_ACCEL_POLLING) {
        Get_Accelerometer_Samples();
    } //if (EAStatus.flags&ETHACCEL_STATUS_ACCEL_POLLING) {

    //I was processing interrupts in the ext int isrs,
    //but UDP packets are sent too quickly
    //Now interrupts are processed the same way polling samples are
    //because we are only checking for an interrupt every timer interrupt callback
    //interval (10ms))
    if (EAStatus.flags&ETHACCEL_STATUS_ACCEL_INTERRUPT) {
        //Process_Accelerometer_Interrupt();
        Get_Accelerometer_Samples();
        
    } //if (EAStatus.flags&ETHACCEL_STATUS_ACCEL_INTERRUPT) {

    if ((EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_POLLING) ||
        (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT)) {
        //IFS1bits.AD1IF=0; //clear AD1 interrupt flag
        //pic32mx IEC1bits.AD1IE=1; //enable AD1 interrupt
        //ADCCON2bits.EOSIEN=1; //enable ADC7 End of Scan interrupt
        IEC6bits.ADCEOSIE=1; //enable end of scan interrupt
        //AD1CON1bits.ASAM=1; //autostart sampling        
        ADCCON3bits.GSWTRG = 1;// Trigger a conversion
    } //
    //IFS0bits.T3IF=0; //clear interrupt flag
    //pic32mx IFS0CLR=_IFS0_T3IF_MASK; //clear interrupt flag- atomic- single instruction
}
  
void __ISR(_UART3_TX_VECTOR, ipl3AUTO) _IntHandlerDrvUsartTransmitInstance0(void)
{
    DRV_USART_TasksTransmit(sysObj.drvUsart0);
}
void __ISR(_UART3_RX_VECTOR, ipl3AUTO) _IntHandlerDrvUsartReceiveInstance0(void)
{
    DRV_USART_TasksReceive(sysObj.drvUsart0);
}
void __ISR(_UART3_FAULT_VECTOR, ipl2AUTO) _IntHandlerDrvUsartErrorInstance0(void)
{
    DRV_USART_TasksError(sysObj.drvUsart0);
}
 
 

 
void __ISR(_UART1_TX_VECTOR, ipl3AUTO) _IntHandlerDrvUsartTransmitInstance1(void)
{
    DRV_USART_TasksTransmit(sysObj.drvUsart1);
}
void __ISR(_UART1_RX_VECTOR, ipl3AUTO) _IntHandlerDrvUsartReceiveInstance1(void)
{
    DRV_USART_TasksReceive(sysObj.drvUsart1);
}
void __ISR(_UART1_FAULT_VECTOR, ipl2AUTO) _IntHandlerDrvUsartErrorInstance1(void)
{
    DRV_USART_TasksError(sysObj.drvUsart1);
}
 
 

 

 

 

 
 

     
void __ISR(_TIMER_9_VECTOR, ipl6AUTO) _IntHandlerDrvI2CMasterInstance0(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_9);
	DRV_I2C_BB_Tasks(sysObj.drvI2C0);
}
      

    
     
   
  
void __ISR(_TIMER_8_VECTOR, ipl6AUTO) _IntHandlerDrvI2CMasterInstance1(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_8);
    DRV_I2C_BB_Tasks(sysObj.drvI2C1);
}
      
    


     
void __ISR(_TIMER_7_VECTOR, ipl6AUTO) _IntHandlerDrvI2CMasterInstance2(void)
{
    PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_TIMER_7);
	DRV_I2C_BB_Tasks(sysObj.drvI2C2);
}
      

    

     
     
   
  
 

  
  
  
 void __ISR(_ETHERNET_VECTOR, ipl5AUTO) _IntHandler_ETHMAC(void)
{
    DRV_ETHMAC_Tasks_ISR((SYS_MODULE_OBJ)0);
}

/* This function is used by ETHMAC driver */
bool SYS_INT_SourceRestore(INT_SOURCE src, int level)
{
    if(level)
    {
        SYS_INT_SourceEnable(src);
    }

    return level;
}


void __ISR(_ADC_EOS_VECTOR,IPL4AUTO)  ADC_ISR(void)   //IPL=Interrupt Priority Level
{    
    uint8_t i,temp,ActiveSensor;
    uint32_t TouchSensorMask;
    uint16_t Sample,LastSample,Threshold,TouchSensorRange;
    float PercentPress,PercentChange;
    //uint8_t FoundChange;
    uint8_t SendTouchSensorPacket;
    uint32_t IntStatus;


    //SYS_CONSOLE_PRINT("adc int\r\n");  

    IEC6bits.ADCEOSIE=0; //disable end of scan interrupt
    IntStatus = ADCCON2; //must read reg to stop clear interrupt cause and stop interrupt from being entered again

    
    //todo: probably I should clear the global polling flag if no accles are being polled
    
    //#define TOUCH_THRESHOLD 11//9
    //9: detects even the tiniest touch, but also detects pressure from motion
    //11: pretty much doesn't detect motion or the tiniest touch, but detects very subtle touches

    //polling and a single sample are the same code
    //pic32mx IEC1bits.AD1IE=0; //disable (this) AD1 interrupt (otherwise it occurs so fast that other CPU tasks fail- for example - the PIC can't get any new instructions)
    //ADCCON2bits.EOSIEN=0; //disable (this) End of Scan interrupt (otherwise it occurs so fast that other CPU tasks fail- for example - the PIC can't get any new instructions)
    
    //AD1CON1bits.ASAM=0; //stop sampling
    //SYS_CONSOLE_PRINT("ADCCON1:%x\r\n",ADCCON1);
    //SYS_CONSOLE_PRINT("ADCCON2:%x\r\n",ADCCON2);
    //SYS_CONSOLE_PRINT("ADCCON3:%x\r\n",ADCCON3);
    
    TouchSensorSendLen=5;
    SendTouchSensorPacket=0;

    
    //SYS_CONSOLE_PRINT("0:%d\r\n",ADCDATA0);    
    //SYS_CONSOLE_PRINT("1:%d\r\n",ADCDATA1);    
    //SYS_CONSOLE_PRINT("4:%d\r\n",ADCDATA4);
    //SYS_CONSOLE_PRINT("5:%d\r\n",ADCDATA5);  
    //SYS_CONSOLE_PRINT("7:%d\r\n",ADCDATA7);    
    //SYS_CONSOLE_PRINT("8:%d\r\n",ADCDATA8);    
    //SYS_CONSOLE_PRINT("CSS1:%x\r\n",ADCCSS1);  
    //SYS_CONSOLE_PRINT("CSS1:%d\r\n",ADCCSS1);  



    //if we are in this ISR, there must be at least 1 active touch sensor
    //go through all active sensors
    //determine if this sample is the new min or max
    //save this sample
    //determine the percent the touch sensor is being pressed
    //determine the change (+ or -) in percent since the last press
    //if the touch sensor is set for interrupt mode, 
    //then determine if enough change has occurred, 
    //if yes then send the above info
    //otherwise if getting one sample or polling send the above data every time
    for(i=0;i<NumActiveTouchSensors;i++) {
        ActiveSensor=ActiveTouchSensor[i];     
        Sample=(uint16_t)*TouchSensor[ActiveSensor].ADCBuf;
        //SYS_CONSOLE_PRINT("S:%d %d %d\r\n",i,ActiveSensor,Sample);  
        //compare this sample to the last one
        if (TouchSensor[ActiveSensor].flags&TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE) {        
            LastSample=TouchSensor[ActiveSensor].LastSample;
        } else {
            LastSample=Sample; //use this sample
            //if not interrupting set flag now (if interrupting flag will get set below))
            //remember that we have seen the first sample for this sensor
            TouchSensor[ActiveSensor].flags|=TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE;
        }
        //remembering the sample here- means that the change must occur 
        //within the time of a single timer tick
        //storing this sample as the last sample below, 
        //means that the change may occur over a longer time 
        //(since the last major change)
        TouchSensor[ActiveSensor].LastSample=Sample; //remember this sample

        //determine if this sample is the new max or min
        if (Sample<TouchSensor[ActiveSensor].Min) {
            TouchSensor[ActiveSensor].Min=Sample;
        }
        if (Sample>TouchSensor[ActiveSensor].Max) {
            TouchSensor[ActiveSensor].Max=Sample;
        }


        if (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT) {
            //touch sensor is configured for interrupt mode
            //make sure there has already been at least one sample to compare to
            //if (TouchSensor[ActiveSensor].flags&TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE) {
                Threshold=TouchSensor[ActiveSensor].Threshold;
                if ((Sample<(LastSample-Threshold)) || (Sample> (LastSample+Threshold))) {
                    //The difference between this sample and the last sample is over the threshold
                    SendTouchSensorPacket=1; //calculate and send the info for this sensor
                } //if (Sample+Threshold <LastSample || Sample-Threshold>LastSample) {
            //} else { //if (TouchSensor[ActiveSendor].flags&TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE) {
                //remember that we have seen the first sample for this sensor
            //    TouchSensor[ActiveSensor].flags|=TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE;
                
           // } //if (TouchSensor[ActiveSendor].flags&TOUCH_SENSOR_STATUS_GOT_INITIAL_SAMPLE) {    
        } else { //if (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT) {
            //touch sensor is not configured for an interrupt so 
            //it is configured to get a single sample or is polling
            SendTouchSensorPacket=1;

        } ////if (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT) {
        
        
        if (SendTouchSensorPacket) {
            //we need to send a TouchSensorPacket so calculate 
            //everything we need to send.
            
            //add this touch sensor data to the Touch Sensor data packet
            //Touch Sensor data packet contains:
            //TouchSensor# [0), %Press [1], %Change [2], Sample [3-4]
            //TouchSensor# (1 byte))
            *(TouchSensorSend+TouchSensorSendLen++)=ActiveTouchSensor[i];
            //%Press (touch sensor is being pressed)
            TouchSensorRange=(TouchSensor[ActiveSensor].Max-TouchSensor[ActiveSensor].Min);
            PercentPress=(((float)Sample-(float)TouchSensor[ActiveSensor].Min)/(float)TouchSensorRange);
            //check for over limit
            if (PercentPress>1.0) {
                PercentPress=1.0;
            }
            *(TouchSensorSend+TouchSensorSendLen++)=(uint8_t)(PercentPress*0xff);
            //%Change ()
            PercentChange=(((float)Sample-(float)LastSample)/(float)TouchSensorRange);
            //check for over and under limit            
            if (PercentChange>1.0) {
                PercentChange=1.0;
            }
            if (PercentChange<-1.0) {
                PercentChange=-1.0;
            }
            *(TouchSensorSend+TouchSensorSendLen++)=(uint8_t)(PercentChange*0xff);                    
            //Sample (voltage)
            memcpy(TouchSensorSend+TouchSensorSendLen,(uint16_t *)&Sample,2);
            TouchSensorSendLen+=2;            
        } //if (SendTouchSensorPacket))
        
            
    } //for i
    
    
    
    if (SendTouchSensorPacket) {

        //at least one touch sensor has to send data
        //so send the Touch Sensor Packet (contains samples for all touch sensors))
        if (TouchSensorSendLen>5) {
            //there was at least one touch sensor change larger than the threshold
            //send all the samples
            while (TCPIP_UDP_PutIsReady(appData.socket)<TouchSensorSendLen) {};
            TCPIP_UDP_ArrayPut(appData.socket,TouchSensorSend,TouchSensorSendLen);  //little endian       
            TCPIP_UDP_Flush(appData.socket); //send the packet                                                    
            //AD1CON1bits.ASAM=1; //resume sampling
        } //if (TouchSensorSendLen>5) {

    } //if (SendTouchSensorPacket) {
        
            
    //if (EAStatus.flags&(ETHACCEL_STATUS_TOUCH_SENSOR_POLLING|ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT)) {
    //if (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT) {
    //timer sets - probably do not need
    //pic32mx AD1CON1bits.ASAM=1; //resume sampling
        //ADCCON3bits.GSWTRG = 1;// Trigger a conversion
    //} //if (EAStatus.flags&ETHACCEL_STATUS_TOUCH_SENSOR_INTERRUPT) {
    //IFS1bits.AD1IF=0; //clear interrupt- needs to be done early or else exceptions ccur -perhaps because other interrupts need to be serviced?
    //IFS1CLR=_IFS1_AD1IF_MASK;
    /* Clear ADC Interrupt Flag */
    //PLIB_INT_SourceFlagClear(INT_ID_0, INT_SOURCE_ADC_END_OF_SCAN);
    IFS6bits.ADCEOSIF=0; //clear end of scan interrupt flag
    
}	//void __ISR(_ADC_EOS_VECTOR,ipl4) ADC_ISR(void) {  //IPL=Interrupt Priority Level

 

/*******************************************************************************
 End of File
*/

